/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "messagedispatcher.h"
#include "messagefactory.h"
#include "events.h"

#include <private/qucom_p.h>
#include <qmetaobject.h>
#include <qregexp.h>
#include <qapplication.h>

using namespace IRC;

namespace
{
    QString messageClassString( const QString &str )
    {
        QString result = str.lower();
        result[ 0 ] = result[ 0 ].upper();
        return result + "Message";
    }
}

MessageDispatcher::MessageDispatcher( QObject *parent, const char *name )
    : QObject( parent, name )
{
    m_typeToReceiver.setAutoDelete( true );
}

void MessageDispatcher::registerMessageReceiver( QObject *receiver, MessageReceiverType receiverType )
{
    qDebug( "registerMessageReceiver: %p(%s)", ( void* )receiver, receiver->className() );

    QStringList types = handledMessageTypesForReceiver( receiver );

    QStringList::ConstIterator it = types.begin();
    for ( ; it != types.end(); ++it ) 
        registerMessageReceiver( *it, receiver, receiverType );
}

void MessageDispatcher::registerMessageReceiver( const QString &key, QObject *receiver, MessageReceiverType receiverType )
{
    QObjectList *receivers = m_typeToReceiver[ key ];
    if ( !receivers ) {
        receivers = new QObjectList;
        m_typeToReceiver.insert( key, receivers );
    }

    if ( receivers->findRef( receiver ) != -1 ) 
        return;

    // let existing handlers finish first..
    //receivers->prepend( receiver );
    receivers->append( receiver );
    qDebug( "registered handler for %s", key.ascii() );

    if ( !m_receivers.contains( receiver ) ) {
        m_receivers.append( receiver );
        connect( receiver, SIGNAL( destroyed( QObject * ) ),
                 this, SLOT( receiverDestroyed( QObject * ) ) );
    }

    if ( receiverType == TemporaryMessageReceiver ) {
        if ( static_cast<QObject *>( m_temporaryReceiver ) != receiver ) {
            qDebug( "changing temporary receiver" );
            if ( m_temporaryReceiver ) {
                qDebug( "unregistering old temporary receiver %s", m_temporaryReceiver->className() );
                unregisterMessageReceiver( m_temporaryReceiver );
            }

            m_temporaryReceiver = receiver;
            m_keysOfTemporaryReceiver.clear();
        }

        m_keysOfTemporaryReceiver << key;
    }
}

QStringList MessageDispatcher::handledMessageTypesForReceiver( QObject *receiver )
{
    return messageTypesFromSlots( receiver->metaObject()->slotNames( true /* superClasses */ ) );
}

void MessageDispatcher::unregisterMessageReceiver( QObject *receiver )
{
    bool knownListener = unregisterMessageReceiverDirect( receiver );

    if ( knownListener ) {
        MessageReceiverRegistrationEvent ev( false /*registeredHandler*/ );
        QApplication::sendEvent( receiver, &ev );
    }
}

void MessageDispatcher::dispatchMessage( const IRC::Message &msg )
{
    dispatchMessage( msg.type(), msg );
}

void MessageDispatcher::dispatchMessage( const QString &key, const IRC::Message &body )
{
    qDebug( "dispatching %s (type %s)", key.ascii(), body.type().ascii() );
    QObjectList *receiverList = m_typeToReceiver[ key ];
    if ( !receiverList || receiverList->isEmpty() ) {
        emit unhandledMessage( body );
        return;
    }

    if ( m_temporaryReceiver ) {
        bool isKeyOfTemporaryReceiver = m_keysOfTemporaryReceiver.findIndex( key ) != -1;

        if ( !isKeyOfTemporaryReceiver ) {
            qDebug( "unregistering temporary receiver" );
            unregisterMessageReceiver( m_temporaryReceiver );
            m_temporaryReceiver = 0;
        }
    }

    QObject *receiver = receiverList->getFirst();

    QString type = body.type();

    Message *msg;
    if ( body.isNumericType() )
        msg = new Message( body );
    else
        msg = MessageFactory::self().createMessage( type.ascii(), body );

    QString slotName = slotForMessage( body );

    QMetaObject *metaObject = receiver->metaObject();

    int slot = metaObject->findSlot( slotName.ascii(), true /*superClasses*/ );
    if ( slot == -1 ) {
        emit unhandledMessage( body );
        return;
    }

    QUObject parameters[ 2 ];
    static_QUType_ptr.set( parameters + 1, msg );
    receiver->qt_invoke( slot, parameters );

    delete msg;
}

void MessageDispatcher::receiverDestroyed( QObject *receiver )
{
    unregisterMessageReceiverDirect( receiver );
}

QStringList MessageDispatcher::messageTypesFromSlots( const QStrList &slotNames )
{
    QStringList typeList;

    QStrListIterator it( slotNames );
    for ( ; it.current(); ++it ) {
        QString type = messageTypeFromSlot( QString::fromLatin1( it.current() ) );
        if ( !type.isEmpty() )
            typeList << type;
    }

    return typeList;
}

QString MessageDispatcher::messageTypeFromSlot( const QString &slot )
{
    // QRegExp is smart enough to use its internal cache to avoid compiling the regexp
    // everytime from scratch.
    QRegExp messageHandlerRegExp( "handle(.+)Message\\(const IRC::(.+)Message&\\)" );

    int pos = messageHandlerRegExp.search( slot, 0 );
    if ( pos != 0 ) 
        return QString::null;

    if ( static_cast<unsigned int>( messageHandlerRegExp.matchedLength() ) != slot.length() ) 
        return QString::null;

    QStringList capturedTexts = messageHandlerRegExp.capturedTexts();
    if ( capturedTexts.count() != 3 ||
         capturedTexts[ 1 ] != capturedTexts[ 2 ] ) 
        return QString::null;

    return capturedTexts[ 1 ].upper();
}

QString MessageDispatcher::slotForMessage( const Message &msg )
{
    QString type = msg.type();

    QString messageClassName = messageClassString( type );

    if ( msg.isNumericType() )
        return QString::fromLatin1( "handleNumericMessage(const IRC::Message&)" );

    return QString::fromLatin1( "handle%1(const IRC::%2&)" ).arg( messageClassName ).arg( messageClassName );
}

bool MessageDispatcher::unregisterMessageReceiverDirect( QObject *receiver )
{
    QDictIterator<QObjectList> it( m_typeToReceiver );
    while ( it.current() ) {
        QObjectListIt listIt( *it.current() );
        while ( listIt.current() ) {
            if ( listIt.current() == receiver ) {
                qDebug( "disconnect %s", it.currentKey().ascii() );
                it.current()->removeRef( receiver );
            }
            else
                ++listIt;
        }

        if ( it.current()->isEmpty() )
            m_typeToReceiver.remove( it.currentKey() );
        else
            ++it;
    }

    bool knownListener = m_receivers.removeRef( receiver );

    disconnect( receiver, SIGNAL( destroyed( QObject * ) ),
                this, SLOT( receiverDestroyed( QObject * ) ) );

    return knownListener;
}

#include "messagedispatcher.moc"

/* vim: et sw=4 ts=4
 */
