/* This file is part of the KDE project
   Copyright (C) 2002 Simon Hausmann <hausmann@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "window.h"
#include "frontend.h"
#include "commands.h"
#include "messagefilter.h"
#include "ctcp.h"

#include <qtimer.h>

#include <klocale.h>

#include <assert.h>

using namespace IRC;

Window::Window( const QString &channel, Frontend *parent, const char *name )
    : QObject( parent, name ), m_channel( channel ), m_frontend( parent ), m_type( PublicChannel )
{
}

Window::~Window()
{
    for ( PersonList::ConstIterator personIt = m_users.begin();
          personIt != m_users.end(); ++personIt )
        m_frontend->userLeftChannel( ( *personIt ).nick(), this );

    emit windowDestroyed( this );
}

void Window::handlePrivmsgMessage( const IRC::PrivmsgMessage &msg )
{
    if ( msg.isCTCPMessage() ) {
        m_frontend->ctcpHandler()->handleCTCPMessage( msg.plainText(), msg, this );
        return;
    }

    printMessage( msg.text() );
}

void Window::handleNoticeMessage( const IRC::NoticeMessage &msg )
{
    handlePrivmsgMessage( msg );
}

void Window::handleTopicMessage( const IRC::TopicMessage &msg )
{
    setTitle( msg.topic() );
}

void Window::handleJoinMessage( const IRC::JoinMessage &msg )
{
    printMessage( msg.description( m_channel ) );

    QString nick = msg.sender().nick();

    if ( nick == m_frontend->connectionInfo().nick() ) {
        ReplyHandler *handler = new NamesReplyHandler( m_channel, this );
        connect( handler, SIGNAL( channelUsers( const IRC::NamesReplyHandler::ChannelUsersMap & ) ),
                 this, SLOT( updateUsersListFromNamesReply( const IRC::NamesReplyHandler::ChannelUsersMap & ) ) );

        m_frontend->installReplyHandler( NamesMessage::name(), handler, MessageDispatcher::PermanentMessageReceiver );

        QTimer::singleShot( INITIAL_USERLIST_UPDATE_TIMEOUT, this, SLOT( updateUserList() ) );

        QTimer *timer = new QTimer( this );
        connect( timer, SIGNAL( timeout() ),
                 this, SLOT( updateUserList() ) );
        timer->start( USERLIST_UPDATE_INTERVAL );
    }

    assert( m_users.findIndex( msg.sender() ) == -1 );

    addUser( nick );
}

void Window::handlePartMessage( const IRC::PartMessage &msg )
{
    printMessage( msg.description( m_channel ) );

    QString nick = msg.sender().nick();

    m_frontend->userLeftChannel( nick, this );

    removeUser( nick );

    // not particularly beautiful...
    if ( nick == m_frontend->connectionInfo().nick() ) 
        deleteLater();
}

void Window::handleQuitMessage( const IRC::QuitMessage &msg )
{
    printMessage( msg.description() );
    removeUser( msg.sender().nick() );
}

void Window::handleNickMessage( const IRC::NickMessage &msg )
{
    PersonList::ConstIterator it = m_users.find( Person( msg.nick() ) );
    if ( it == m_users.end() )
        return;

    printMessage( msg.description() );

    emit userUpdated( msg.sender(), ( *it ) );
}

void Window::handleNumericMessage( const IRC::Message &msg )
{
    switch ( msg.numericType() ) {
        case RPL_TOPIC:
            setTitle( msg.parameter( 2 ) );
            break;
        case RPL_NOTOPIC:
            setTitle( i18n( "No topic is set" ) );
            break;
    }
}

void Window::updateUsersListFromNamesReply( const IRC::NamesReplyHandler::ChannelUsersMap &users )
{
    for ( NamesReplyHandler::ChannelUsersMap::ConstIterator it = users.begin();
          it != users.end(); ++it ) {

        if ( it.key().name != m_channel )
            continue;

        m_type = it.key().type;
        updateUsers( *it );
    }
}

void Window::processInput( const QString &text )
{
    if ( text[ 0 ] == '/' ) {
        processCommand( text.mid( 1 ) );
        return;
    }

    PrivmsgMessage msg( m_frontend->createPrefixedMessage<PrivmsgMessage>() );
    msg.setReceiver( channel() ); 
    msg.setPlainText( text );
    emit sendOneWayMessage( msg );
    handlePrivmsgMessage( msg );
}

void Window::processCommand( const QString &text )
{
    //QStringList arguments = QStringList::split( " ", text );
    QStringList arguments = splitStringListObeyingQuotes( ' ', text );

    QString command = arguments.front().upper();
    arguments.pop_front();

    processCommand( command, arguments );
}

void Window::processCommand( const QString &command, const QStringList &arguments )
{
    Message msg = m_frontend->createPrefixedMessage( command, arguments );
    processCommand( msg );
}

void Window::processCommand( const Message &msg )
{
    Message filteredMessage( msg );

    if ( m_frontend->messageFilter().filterMessage( filteredMessage, this ) == MessageFilter::DiscardMessage )
        return;

    Command *cmd = new UserCommand( filteredMessage, this );
    emit executeCommand( cmd );
}

void Window::updateUserList()
{
    NamesCommand *command = new NamesCommand( m_channel, this );
    connect( command, SIGNAL( channelUsers( const IRC::NamesReplyHandler::ChannelUsersMap & ) ),
             this, SLOT( updateUsersListFromNamesReply( const IRC::NamesReplyHandler::ChannelUsersMap & ) ) );
    emit executeCommand( command );
}

void Window::updateUsers( const NamesReplyHandler::NickInfoList &users )
{
    bool needDetails = ( m_users.count() == 1 ); // only us?

    for ( NamesReplyHandler::NickInfoList::ConstIterator it = users.begin();
          it != users.end(); ++it )
        updateUser( *it );

    if ( needDetails ) {
        PersonList updateList;

        for ( PersonList::ConstIterator it = m_users.begin();
              it != m_users.end(); ++it )
            if ( ( *it ).channels().isEmpty() )
                updateList << *it;

        if ( !updateList.isEmpty() )
            m_frontend->updateUserDetails( updateList );
    }
}

void Window::updateUser( const NamesReplyHandler::NickInfo &user )
{
    PersonList::Iterator userIt = m_users.find( Person( user.name ) );
    if ( userIt == m_users.end() )
        userIt = addUser( user.name );

    ( *userIt ).channels()[ m_channel ].isChannelOperator = user.isOp;
}

PersonList::Iterator Window::addUser( const QString &nick )
{
    Person person = m_frontend->userJoinedChannel( nick, this );
    PersonList::Iterator it = m_users.append( person );

    emit userUpdated( Person(), person );

    m_frontend->updateUserDetails( nick );

    return it;
}

void Window::removeUser( const QString &nick )
{
    PersonList::Iterator personIt = m_users.find( Person( nick ) );
    if ( personIt == m_users.end() )
        return;

    emit userRemoved( *personIt );

    m_users.remove( personIt );
}

#include "window.moc"

/* vim: et sw=4 ts=4
 */
